@@ -1,8 +1,8 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: awk.c,v 1.24 2004/10/06 05:17:41 shadow Exp $
+ * $Id: awk.c,v 1.32 2005/08/30 20:52:21 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2004  The Xastir Group
+ * Copyright (C) 2003-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -64,11 +64,21 @@
 #ifdef HAVE_LIBPCRE
 #include <stdio.h>
 #include <string.h>
+
+// Needed for Solaris
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif  // HAVE_STRINGS_H
+
 #include <ctype.h>
 #include <sys/types.h>
 #include "awk.h"
 #include "snprintf.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 #define min(a,b) ((a)<(b)?(a):(b))
 
@@ -94,7 +104,7 @@
 /*
  * awk_new_symtab: alloc a symbol table with $0-$9 pre-declared.
  */
-awk_symtab *awk_new_symtab() {
+awk_symtab *awk_new_symtab(void) {
     awk_symtab *n = calloc(1,sizeof(awk_symtab));
     static char sym[MAXSUBS][2];
     int i;
@@ -179,13 +189,16 @@ awk_symbol *awk_find_sym(awk_symtab *this,
                  const char *name,
                  const int len) {
     awk_symbol *s;
-    char c, d;
+    char c;
 
 
-    // Create holding spot for both cases of first character, in
-    // order to speed up the loop below.
-    c = toupper(name[0]);
-    d = name[0];
+    // Create holding spot for first character in order to speed up
+    // the loop below.  Note that "toupper()" is very slow on some
+    // systems so we took it out of this function to speed things
+    // up.  We evidently don't need case insensitive behavior here
+    // anyway.
+    //
+    c = name[0];
 
     // Check through the hash
     //
@@ -194,12 +207,14 @@ awk_symbol *awk_find_sym(awk_symtab *this,
         // Check length first (fast operation)
         if (s->namelen == len) {
 
-            // Check first chars next (fast operation)
-            if (s->name[0] == c || s->name[0] == d) {
+            // Check first char next (fast operation)
+            if (s->name[0] == c) {
 
                 // Ok so far, test the entire string (slow
-                // operation)
-                if (strncmp(s->name,name,len) == 0)
+                // operation, case sensitive)
+                if (len == 1)
+                    return s;
+                if (len > 1 && strncmp(s->name+1,name+1,len-1) == 0)
                     return s;
             }
         }
@@ -348,7 +363,7 @@ int awk_compile_stmt(awk_symtab *this,
              int len) {
     const char *s = stmt, *op, *ep;
   
-    while (isspace(*s)) {               /* clean up leading white space */
+    while (isspace((int)*s)) {               /* clean up leading white space */
         ++s;
         --len;
     }
@@ -356,12 +371,12 @@ int awk_compile_stmt(awk_symtab *this,
 
     if ((op = strchr(s,'=')) != NULL) {   /* it's either an assignment */
         const char *val = op+1;
-        while (isspace(*val)) {
+        while (isspace((int)*val)) {
             val++;
             len--;
         }
         --op;
-        while (isspace(*op) && op>s) op--;
+        while (isspace((int)*op) && op>s) op--;
         p->opcode = ASSIGN;
         p->dest = awk_find_sym(this,s,(op-s+1));
         if (!p->dest) {
@@ -372,7 +387,7 @@ int awk_compile_stmt(awk_symtab *this,
     } else {                    /* or the "next" keyword */
         const char *r;
 
-        for (r=&s[len-1]; isspace(*r); r--,len--) ; /* trim trailing white space */
+        for (r=&s[len-1]; isspace((int)*r); r--,len--) ; /* trim trailing white space */
         if (len == 4 && strncmp(s,"next",4) == 0) {
             p->opcode = NEXT;
         } else if (len == 4 && strncmp(s,"skip",4) == 0) {
@@ -497,7 +512,7 @@ void awk_eval_expr(awk_symtab *this,
                 /* now search for the var name using closing delim */
                 symname = expr;
                 if (delim == '\0') {    /* no close delim */
-                    while (!isspace(*expr) && !ispunct(*expr) && exprlen > 0) {
+                    while (!isspace((int)*expr) && !ispunct((int)*expr) && exprlen > 0) {
                         ++expr;
                         --exprlen;
                     }
@@ -631,7 +646,7 @@ int awk_exec_action(awk_symtab *this, const awk_action *code) {
 /*
  * awk_new_rule: alloc a rule
  */
-awk_rule *awk_new_rule() {
+awk_rule *awk_new_rule(void) {
     awk_rule *n = calloc(1,sizeof(awk_rule));
 
     if (!n)
@@ -672,7 +687,7 @@ void awk_free_rule(awk_rule *r) {
 /*
  * awk_new_program: alloc a program
  */
-awk_program *awk_new_program() {
+awk_program *awk_new_program(void) {
     awk_program *n = calloc(1,sizeof(awk_program));
 
     if (!n)
@@ -799,7 +814,7 @@ awk_program *awk_load_program_file(const char *file) { /* rules filename */
         ++line;
         if (in[l-1] == '\n')
             in[--l] = '\0';
-        while (isspace(*cp)) ++cp;
+        while (isspace((int)*cp)) ++cp;
         switch(*cp) {
         case '\0':              /* empty line */
             continue;
@@ -852,7 +867,7 @@ awk_program *awk_load_program_file(const char *file) { /* rules filename */
             garbage(file,line,in,cp);
             continue;
         }
-        while (isspace(*cp)) ++cp; /* skip whitespace */
+        while (isspace((int)*cp)) ++cp; /* skip whitespace */
         if (*cp == '{') {
             p = ++cp;
         loop: 
@@ -877,7 +892,7 @@ awk_program *awk_load_program_file(const char *file) { /* rules filename */
 
             r->flags |= AR_MALLOC;
             /* make sure there's no extraneous junk on the line */
-            while (*cp && isspace(*cp)) ++cp;
+            while (*cp && isspace((int)*cp)) ++cp;
             if (*cp == '#' || *cp == '\0')
                 awk_add_rule(n,r);
             else {
