@@ -1,5 +1,5 @@
 /*
- * $Id: bulletin_gui.c,v 1.30 2005/07/31 06:19:11 we7u Exp $
+ * $Id: bulletin_gui.c,v 1.34 2005/10/02 06:26:49 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
@@ -32,7 +32,12 @@
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
+
+// Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
+
 #include <ctype.h>
 #include <sys/types.h>
 
@@ -69,6 +74,9 @@
 #include "interface.h"
 #include "util.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 Widget Display_bulletins_dialog = NULL;
@@ -444,8 +452,8 @@ static void find_zero_position_bulletins(void) {
 // that are newer than first_new_bulletin_time and fit within our
 // range.  If any found, it updates the Bulletins dialog.
 time_t last_bulletin_check = (time_t)0l;
-void check_for_new_bulletins() {
 
+void check_for_new_bulletins(void) {
 
     // Check every 15 seconds max
     if ( (last_bulletin_check + 15) > sec_now() ) {
@@ -586,7 +594,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
 
         Display_bulletins_dialog = XtVaCreatePopupShell(langcode("BULMW00001"),
                 xmDialogShellWidgetClass,
-                Global.top,
+                appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
