@@ -1,9 +1,9 @@
-/* -*- c-basic-indent: 4; indent-tabs-mode: nil -*-
- * $Id: color.c,v 1.5 2003/02/04 04:08:36 jtwilley Exp $
+/* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
+ * $Id: color.c,v 1.14 2005/10/20 20:34:32 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2003  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,11 +32,20 @@
 #include "color.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 static color_load color_choice[MAX_COLORS];
 static int colors_loaded;
 static int rm, gm, bm; // rgb masks
 static int rs, gs, bs; // rgb shifts
+int visual_depth;
+
+
+
+
 
 /**********************************************************************************/
 /* load color file                                                                */
@@ -66,7 +75,12 @@ int load_color_file(void) {
                         }
                     }
                     if (ok) {
-                        strcpy(color_choice[colors_loaded].colorname,colorname);
+                       
+                        xastir_snprintf(color_choice[colors_loaded].colorname,
+                            MAX_COLORNAME,
+                            "%s",
+                            colorname);
+
 // Do we really want to assign to unsigned short int's here?
                         color_choice[colors_loaded].color.red=(unsigned short)(r*257);
                         color_choice[colors_loaded].color.blue=(unsigned short)(b*257);
@@ -87,6 +101,10 @@ int load_color_file(void) {
     return(ok);
 }
 
+
+
+
+
 /**********************************************************************************/
 /* GetPixelbyName                                                                 */
 /* get color for the named choice                                                 */
@@ -101,6 +119,7 @@ Pixel GetPixelByName( Widget w, char *colorname) {
 
     found=-1;
     i=0;
+
     do {
         if (strcmp(color_choice[i].colorname, colorname)==0)
             found=i;
@@ -109,20 +128,31 @@ Pixel GetPixelByName( Widget w, char *colorname) {
     } while (i<colors_loaded && found <0);
 
     if (found >= 0) {
-        if (XAllocColor(dpy,cmap,&color_choice[found].color))
+
+        // XFreeColors() here generates an error.  Why?
+        // "BadAccess (attempt to access private resource denied)"
+        // XFreeColors(dpy, cmap, &(color_choice[found].color.pixel),1,0);
+
+        if (XAllocColor(dpy,cmap,&color_choice[found].color)) {
             return(color_choice[found].color.pixel);
+        }
         else {
             xastir_snprintf(warning, sizeof(warning), "Couldn't allocate color %s", colorname);
             XtWarning(warning);
             return(BlackPixel(dpy,scr));
         }
-    } else {
+    }
+    else {
         xastir_snprintf(warning, sizeof(warning), "Couldn't find color %s", colorname);
         XtWarning(warning);
         return(BlackPixel(dpy,scr));
     }
 }
 
+
+
+
+
 void setup_visual_info(Display* dpy, int scr) {
     int visuals_matched, i, j;
     XVisualInfo *visual_list, *vp;
@@ -184,6 +214,9 @@ void setup_visual_info(Display* dpy, int scr) {
             }
             else if (debug_level & 16)
                 fprintf(stderr,"\tID:           0x%lx\n", vp->visualid);
+    
+            // Store color depth for use by other routines.
+            visual_depth = vp->depth;
 
             if (debug_level & 16) {
                 fprintf(stderr,"\tScreen:       %d\n",  vp->screen);
@@ -220,8 +253,13 @@ void setup_visual_info(Display* dpy, int scr) {
             }
         }
     }
+    XFree(visual_list);
 }
 
+
+
+
+
 void pack_pixel_bits(unsigned short r, unsigned short g, unsigned short b, unsigned long* pixel)
 {
     switch (visual_type) {
@@ -259,3 +297,5 @@ void pack_pixel_bits(unsigned short r, unsigned short g, unsigned short b, unsig
         break;
     }
 }
+
+
