@@ -1,9 +1,9 @@
 /*
- * $Id: draw_symbols.h,v 1.11 2003/05/21 22:40:11 francais1 Exp $
+ * $Id: draw_symbols.h,v 1.19 2005/08/04 19:34:06 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2003  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,19 +45,18 @@ extern SymbolData symbol_data[];
 
 extern void draw_nice_string(Widget w, Pixmap where, int style, long x, long y, char *text, int bgcolor, int fgcolor, int length);
 extern void clear_symbol_data(void);
-extern void read_symbol_from_file(FILE *f, char *pixels);
-extern void load_pixmap_symbol_file(char *filename);
-extern void insert_symbol(char table, char symbol, char *pixel, int deg, char orient);
+extern void read_symbol_from_file(FILE *f, char *pixels, char table_char);
+extern void load_pixmap_symbol_file(char *filename, int reloading);
+extern void insert_symbol(char table, char symbol, char *pixel, int deg, char orient, int reloading);
 extern char symbol_orient(char *course);
 extern void symbol(Widget w, int ghost,char symbol_table, char symbol_id, char symbol_overlay, Pixmap where, int mask, long x_offset, long y_offset, char rotate);
 
-extern void draw_symbol(Widget w, char symbol_table, char symbol_id,
-char symbol_overlay, long x_lon, long y_lat,char *callsign_text,
-char *alt_text, char *course_text, char *speed_text, char
-*my_distance, char *my_course, char *wx_temp, char* wx_wind, time_t
-sec_heard, int temp_show_last_heard, Pixmap where, char rotate, char area_type);
+extern void draw_WP_line(DataRow *p_station, Pixmap where, Widget w);
+
+extern void draw_symbol(Widget w, char symbol_table, char symbol_id, char symbol_overlay, long x_lon, long y_lat,char *callsign_text, char *alt_text, char *course_text, char *speed_text, char *my_distance, char *my_course, char *wx_temp, char* wx_wind, time_t sec_heard, int temp_show_last_heard, Pixmap where, char rotate, char area_type, char *signpost, char *pmin, char *pmax, int bump_count );
 
 extern void draw_pod_circle(long x_long, long y_lat, double range, int color, Pixmap where);
+extern void draw_aloha_circle(long x_long, long y_lat, double range, int color, Pixmap where);
 extern void draw_phg_rng(long x_long, long y_lat, char *phg, time_t sec_heard, Pixmap where);
 extern void draw_DF_circle(long x_long, long y_lat, char *shgd, time_t sec_heard, Pixmap where);
 extern void draw_wind_barb(long x_long, long y_lat, char *speed, char *course, time_t sec_heard, Pixmap where);
