@@ -1,9 +1,9 @@
 /*
- * $Id: fcc_data.c,v 1.11 2004/09/17 19:56:56 we7u Exp $
+ * $Id: fcc_data.c,v 1.15 2005/09/09 04:46:56 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,6 +48,9 @@
 #include "xa_config.h"
 #include "main.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 
@@ -101,7 +104,10 @@ int build_fcc_index(int type){
         if (file_time(get_data_base_dir(database_name))<=file_time(get_user_base_dir("data/appl.ndx")))
             return(1);
         else {
-            statusline("FCC index old, rebuilding",1);
+            // FCC index old, rebuilding
+            statusline(langcode("STIFCC0100"),1);
+
+            fprintf(stderr,"FCC index is old.  Rebuilding index.\n");
 //            XmTextFieldSetString(text,"FCC index old, rebuilding");
 //            XtManageChild(text);
 //            XmUpdateDisplay(XtParent(text));     // DK7IN: do we need this ???
@@ -155,6 +161,7 @@ int build_fcc_index(int type){
 
 //    XmTextFieldSetString(text,"");
 //    XtManageChild(text);
+
     return(1);
 }
 
@@ -463,7 +470,11 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
                             // location for the callsign.  Return if so.
                             if ( (temp[0] < line[pos]) ||
                                     ( (temp[0] == line[pos]) && (temp[1] < line[pos+1]) ) ) {
-                                popup_message_always("Callsign Search", "Callsign Not Found!");
+
+                                // "Callsign Search", "Callsign Not Found!"
+                                popup_message_always(langcode("STIFCC0101"),
+                                    langcode("STIFCC0102") );
+
                                 //fprintf(stderr,"%c%c\t%c%c\n",temp[0],temp[1],line[pos],line[pos+1]);
                                 (void)fclose(f);
                                 return(0);
