@@ -1,8 +1,8 @@
 // Modification for Xastir CVS purposes
 //
-// Portions Copyright (C) 2000-2004  The Xastir Group
+// Portions Copyright (C) 2000-2005  The Xastir Group
 //
-// $Id: festival.c,v 1.13 2004/09/07 15:39:52 we7u Exp $
+// $Id: festival.c,v 1.16 2005/08/17 19:08:44 we7u Exp $
 //
 // End of modification
 
@@ -96,6 +96,10 @@
 #include "festival.h"
 #include "snprintf.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 FT_Info *info = NULL;
 
@@ -171,6 +175,7 @@ static int festival_socket_open(const char *host, int port) {
 
     if (connect(fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) != 0) {
         fprintf(stderr,"festival_client: connect to server failed\n");
+        (void)close(fd);    // Close the socket
         return(-1);
     }
     return(fd);
