@@ -1,16 +1,16 @@
 
 
-// $Id: hashtable_private.h,v 1.2 2004/11/02 20:37:06 we7u Exp $
+// $Id: hashtable_private.h,v 1.5 2005/08/18 22:11:00 we7u Exp $
 
 
 /* Copyright (C) 2002, 2004 Christopher Clark <firstname.lastname@cl.cam.ac.uk> */
-/* Modified by the Xastir Group */
+/* Portions Copyright (C) 2004-2005  The Xastir Group */
+
 
 #ifndef __HASHTABLE_PRIVATE_CWC22_H__
 #define __HASHTABLE_PRIVATE_CWC22_H__
 
 #include "hashtable.h"
-#include "xastir.h" /* Defines GC_MALLOC/GC_FREE */
 
 
 /*****************************************************************************/
@@ -40,7 +40,7 @@ hash(struct hashtable *h, void *k);
 static inline unsigned int
 indexFor(unsigned int tablelength, unsigned int hashvalue) {
     return (hashvalue % tablelength);
-};
+}
 
 /* Only works if tablelength == 2^N */
 /*static inline unsigned int
