@@ -1,9 +1,9 @@
 /*
- * $Id: igate.c,v 1.41 2004/11/05 16:37:36 we7u Exp $
+ * $Id: igate.c,v 1.49 2005/10/10 16:29:37 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,6 @@
 #include "snprintf.h"
 
 #include <termios.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -45,6 +44,10 @@
 #include "xa_config.h"
 #include "util.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 time_t last_nws_stations_file_time = 0;
 int NWS_stations = 0;
@@ -52,6 +55,10 @@ int max_NWS_stations = 0;
 NWS_Data *NWS_station_data;
 
 
+void load_NWS_stations(char *file);
+int check_NWS_stations(char *call);
+
+
 
 // Struct for holding packet data.  We use dynamically-allocated
 // singly-linked lists.  The last record should have its "next"
@@ -186,6 +193,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
             // Changes needed before parsing code:  Get rid of first
             // part of packet up to the '}' symbol.  After this the
             // generic parsing code will work.
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
 
             c0 = strstr(line, ":}"); // Find start of 3rd party packet
             if (c0 == NULL) {   // Not 3rd party packet
@@ -197,6 +205,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
             // Copy original packet into line2 for later parsing.  We
             // want to keep the '}' character because our own
             // transmissions out RF have that character as well.
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
             if (debug_level & 1024)
                 fprintf(stderr,"3rd party HeardQ: %s\n",line);
 
@@ -560,6 +569,7 @@ void output_igate_net(char *line, int port, int third_party) {
 
     // Check for third party messages.  We don't want to gate these
     // back onto the internet feeds
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
     if (message[0] == '}') {
 
         if (log_igate && (debug_level & 1024) ) {
@@ -755,12 +765,14 @@ end_critical_section(&devices_lock, "igate.c:output_igate_net" );
 
 /****************************************************************/
 /* output data to tnc interfaces                                */
-/* from: type of port heard from                                */
-/* call: call sign heard from                                   */
+/* from: type of port heard from (No! It's the source call!)    */
+/* call: call sign heard from (No! It's the destination call!)  */
 /* line: data to gate to rf                                     */
 /* port: port data came from                                    */
 /****************************************************************/
-void output_igate_rf(char *from, char *call, char *path, char *line, int port, int third_party) {
+void output_igate_rf(char *from, char *call, char *path, char *line,
+        int port, int third_party, char *object_name) {
+
     char temp[MAX_LINE_SIZE+20];
     int x;
     int first = 1;
@@ -848,11 +860,22 @@ void output_igate_rf(char *from, char *call, char *path, char *line, int port, i
         }
         return;
     }
- 
+
+
+     // check to see if the nws-stations file is newer than last read
+    if (last_nws_stations_file_time < file_time(get_user_base_dir("data/nws-stations.txt"))) {
+        last_nws_stations_file_time = file_time(get_user_base_dir("data/nws-stations.txt"));
+        load_NWS_stations(get_user_base_dir("data/nws-stations.txt"));
+        //fprintf(stderr,"NWS Station file time is old\n");
+    }
+
+
     // Check whether the source and destination calls have been
     // heard on local RF.
-    if (!heard_via_tnc_in_past_hour(call)==1        // Haven't heard destination call in previous hour
-            || heard_via_tnc_in_past_hour(from)) {  // Have heard source call in previous hour
+    if (  (   (!object_name && !check_NWS_stations(from)) // Source call not in nws-stations.txt
+            || (object_name && !check_NWS_stations(object_name))) // Object/item not in nws-stations.txt
+        && (!heard_via_tnc_in_past_hour(call)==1        // Haven't heard destination call in previous hour
+            || heard_via_tnc_in_past_hour(from))) {  // Have heard source call in previous hour
 
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
@@ -953,6 +976,8 @@ end_critical_section(&devices_lock, "igate.c:output_igate_rf" );
                         // unproto_igate path
                         output_my_data(line,x,0,0,1,NULL);
 
+//fprintf(stderr, "Igating->RF: %s\n", line);
+
 begin_critical_section(&devices_lock, "igate.c:output_igate_rf" );
 
                     }
@@ -1021,6 +1046,14 @@ void load_NWS_stations(char *file) {
     if (file[0] == '\0')
         return;
 
+    if (NWS_station_data) {
+        free(NWS_station_data);
+        NWS_station_data = NULL;
+    }
+
+    NWS_stations = 0;
+    max_NWS_stations = 0;
+
     f = fopen(file,"r");
     if (f!=NULL) {
         while (!feof(f)) {
@@ -1033,7 +1066,9 @@ void load_NWS_stations(char *file) {
                         // add data
                         // Note:  Size of string variable is 12
                         // bytes, defined in igate.h
-                        (void)sscanf(line,"%11s",NWS_station_data[NWS_stations-1].call);
+                        if (1 != sscanf(line,"%11s",NWS_station_data[NWS_stations-1].call)) {
+                            fprintf(stderr,"load_NWS_stations: sscanf parsing error\n");
+                        }
                         if (debug_level & 1024)
                             fprintf(stderr,"LINE:%s\n",line);
                     }
@@ -1073,6 +1108,9 @@ int check_NWS_stations(char *call) {
     if (call[0] == '\0')
         return(0);
 
+    if (NWS_station_data == NULL)
+        return(0);
+
     if (debug_level & 1024)
         fprintf(stderr,"igate.c::check_NWS_stations %s\n", call);
 
