@@ -1,9 +1,9 @@
 /*
- * $Id: interface.h,v 1.24 2003/04/11 23:59:08 we7u Exp $
+ * $Id: interface.h,v 1.34 2005/08/22 18:45:58 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2003  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,10 +26,20 @@
 #define __XASTIR_INTERFACE_H
 
 #include <termios.h>
-#include <pthread.h>
 #include "util.h"
 #include "xastir.h"
 
+
+// User configurable text for taking a TNC into converse mode.
+// Uncomment only one of the below two lines.
+//
+// Works for most TNC's, including tnc2-ui
+#define CONVERSE_MODE "k"
+//
+// Works for Kantronics KPC-2
+//#define CONVERSE_MODE "conv"
+
+
 #define MAX_DEVICE_NAME 128
 #define MAX_DEVICE_BUFFER 4096
 #define MAX_DEVICE_BUFFER_UNTIL_BINARY_SWITCH 700
@@ -42,6 +52,8 @@
 
 #define DEFAULT_GPS_RETR 0x05 /* CTRL-E */
 
+// Define a 60 second max wait on a serial port (in microseconds)
+#define SERIAL_MAX_WAIT 60000000
 
 // KISS Protocol Special Characters & Commands:
 #define KISS_FEND           0xc0  // Frame End
@@ -59,7 +71,7 @@
 
 
 
-#define MAX_IFACE_DEVICE_TYPES 13
+#define MAX_IFACE_DEVICE_TYPES 14
 
 /* Define Device Types */
 enum Device_Types {
@@ -75,7 +87,8 @@ enum Device_Types {
     DEVICE_SERIAL_TNC_AUX_GPS,  // KB6MER -> KAM XL or other TNC w/GPS on AUX port
     DEVICE_SERIAL_KISS_TNC,     // KISS TNC on serial port (not ax.25 kernel device)
     DEVICE_NET_DATABASE,
-    DEVICE_NET_AGWPE
+    DEVICE_NET_AGWPE,
+    DEVICE_SERIAL_MKISS_TNC     // Multi-port KISS TNC, like the Kantronics KAM
 };
 
 enum Device_Active {
@@ -138,9 +151,11 @@ typedef struct {
 typedef struct {
     int    device_type;                           /* device type                             */
     char   device_name[MAX_DEVICE_NAME+1];        /* device name                             */
+    char   radio_port[3];                         /* port for multi-port TNC's               */
     char   device_host_name[MAX_DEVICE_HOSTNM+1]; /* device host name for network            */
     char   device_host_pswd[MAX_DEVICE_HOSTPW+1]; /* host password also WX device data type  */
     char   device_host_filter_string[201];        /* host filter string                      */
+    char   comment[50];                           /* Local comment or name for port          */
     char   unproto1[50];                          /* unproto path 1 for this port            */
     char   unproto2[50];                          /* unproto path 2 for this port            */
     char   unproto3[50];                          /* unproto path 3 for this port            */
@@ -162,6 +177,7 @@ typedef struct {
     char   txtail[4];                             /* KISS parameter */
     int    fullduplex;                            /* KISS parameter */
     int    relay_digipeat;                        /* If 1: interface should RELAY digipeat */
+    int    init_kiss;				              /* Initialize KISS-Mode on startup */
 } ioparam;
 
 
@@ -203,9 +219,12 @@ extern int data_avail;
 extern void control_interface(Widget w, XtPointer clientData, XtPointer callData);
 extern void dtr_all_set(int dtr);
 extern void interface_status(Widget w);
+extern void update_interface_list(void);
 extern int WX_rain_gauge_type;
 
 /* interface.c */
+extern int is_local_interface(int port);
+extern int is_network_interface(int port);
 extern void send_agwpe_packet(int xastir_interface, int RadioPort, unsigned char type, unsigned char *FromCall, unsigned char *ToCall, unsigned char *Path, unsigned char *Data, int length);
 
 extern unsigned char *incoming_data;
