@@ -1,13 +1,20 @@
 
-// $Id: io-common.c,v 1.1 2004/03/29 23:40:32 we7u Exp $
+// $Id: io-common.c,v 1.4 2005/08/30 20:28:50 kd6zwr Exp $
 
 
 /* Copyright 2002 Daniel Egnor.  See LICENSE.geocoder file.
+ * Portions Copyright (C) 2004-2005  The Xastir Group
  */
 #include "io.h"
-
 #include <ctype.h>
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
+
+
 int io_out_i4(struct io_file *f,int pos,int o) {
         unsigned char x[4];
         x[0] = o >> 24;
@@ -50,9 +57,9 @@ int io_strntoi(const char *str,int len) {
 	int r = 0;
 	int sign = 1;
 	const char * const end = str + len;
-	while (end != str && isspace(*str)) ++str;
+	while (end != str && isspace((int)*str)) ++str;
 	if (end != str && *str == '-') (sign *= -1),++str;
 	if (end != str && *str == '+') ++str;
-	while (end != str && isdigit(*str)) r = *str++ - '0' + 10 * r;
+	while (end != str && isdigit((int)*str)) r = *str++ - '0' + 10 * r;
 	return r * sign;
 }
