@@ -1,9 +1,9 @@
 /*
- * $Id: location_gui.c,v 1.12 2004/09/17 19:56:56 we7u Exp $
+ * $Id: location_gui.c,v 1.16 2005/08/17 19:09:40 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,6 +37,10 @@
 #include "lang.h"
 #include "snprintf.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 Widget location_dialog = (Widget)NULL;
 Widget location_list;
@@ -134,7 +138,9 @@ void location_view(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@
                             "%s",
                             temp_ptr);
                         if (strcmp(location,name)==0) {
-                            (void)sscanf(pos,"%19s %19s %9s", s_lat, s_long, s_sz);
+                            if (3 != sscanf(pos,"%19s %19s %9s", s_lat, s_long, s_sz)) {
+                                fprintf(stderr,"location_view:sscanf parsing error\n");
+                            }
                             map_pos(convert_lat_s2l(s_lat),convert_lon_s2l(s_long),atol(s_sz));
                             done=1;
                         }
@@ -356,7 +362,7 @@ void location_add(/*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPo
 void Jump_location(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     static Widget  pane,form, button_ok, button_add, button_delete, button_cancel, locdata, location_name;
     int n;
-    Arg al[20];           /* Arg List */
+    Arg al[50];           /* Arg List */
     unsigned int ac = 0;           /* Arg Count */
     Atom delw;
 
