@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_WMS.c,v 1.13 2005/07/22 06:08:22 we7u Exp $
+ * $Id: map_WMS.c,v 1.18 2005/10/20 01:29:56 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
@@ -36,7 +36,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -113,6 +115,9 @@
 #undef XASTIR_PACKAGE_VERSION
 #endif // HAVE_IMAGEMAGICK
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 extern int npoints;		/* tsk tsk tsk -- globals */
@@ -537,7 +542,16 @@ void draw_WMS_map (Widget w,
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -715,11 +729,14 @@ void draw_WMS_map (Widget w,
             }
 
             // Get the color allocated on < 8bpp displays. pixel color is written to my_colors.pixel
-            if (visual_type == NOT_TRUE_NOR_DIRECT)
+            if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                 XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-            else
+            }
+            else {
                 pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                 &my_colors[l].pixel);
+            }
 
             if (debug_level & 512)
                 fprintf(stderr,"Color allocated is %li  %i  %i  %i \n", my_colors[l].pixel,
