@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_dos.c,v 1.13 2004/09/18 20:25:47 we7u Exp $
+ * $Id: map_dos.c,v 1.19 2005/09/29 19:19:11 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,7 +35,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -67,6 +69,11 @@
 #include "color.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 #define CHECKMALLOC(m)  if (!m) { fprintf(stderr, "***** Malloc Failed *****\n"); exit(0); }
 
 #define DOS_HDR_LINES 8
@@ -310,12 +317,12 @@ void draw_dos_map(Widget w,
              alert_entry *alert,
              u_char alert_color,
              int destination_pixmap,
-             int draw_filled) {  
+             map_draw_flags *mdf) {  
   FILE *f;
   char file[MAX_FILENAME];
   char short_filenm[MAX_FILENAME];
   char map_it[MAX_FILENAME];
-  
+
   /* map header info */
   char map_type[5];
   char map_version[5];
@@ -377,7 +384,11 @@ void draw_dos_map(Widget w,
   char symbol_id;
   char symbol_color;
   int embedded_object;
+
+  int draw_filled;
   
+  draw_filled=mdf->draw_filled;
+
   x = 0;
   y = 0;
   color = -1;
@@ -574,10 +585,11 @@ void draw_dos_map(Widget w,
       
       // We're indexing only.  Save the extents in the index.
       index_update_xastir(filenm, // Filename only
-                          bottom_boundary,        // Bottom
-                          top_boundary,           // Top
-                          left_boundary,          // Left
-                          right_boundary);        // Right
+                          bottom_boundary,  // Bottom
+                          top_boundary,     // Top
+                          left_boundary,    // Left
+                          right_boundary,   // Right
+                          1000);            // Default Map Level
       
       (void)fclose (f);
 
@@ -596,7 +608,16 @@ void draw_dos_map(Widget w,
         (void)fclose(f);
 
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
    
@@ -631,7 +652,16 @@ void draw_dos_map(Widget w,
           (void)fclose(f);
 
           // Update to screen
-          (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+          (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
           return;
       }
       
@@ -934,7 +964,16 @@ void draw_dos_map(Widget w,
             (void)fclose(f);
 
             // Update to screen
-            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
             return;
         }
 
