@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_pdb.c,v 1.8 2004/09/18 20:25:47 we7u Exp $
+ * $Id: map_pdb.c,v 1.14 2005/09/29 19:19:11 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -68,6 +70,11 @@
 #include "color.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 #define CHECKMALLOC(m)  if (!m) { fprintf(stderr, "***** Malloc Failed *****\n"); exit(0); }
 
 #define DOS_HDR_LINES 8
@@ -92,7 +99,7 @@ void draw_palm_image_map(Widget w,
 			 alert_entry *alert, 
 			 u_char alert_color,
 			 int destination_pixmap, 
-			 int draw_filled) {
+			 map_draw_flags *mdf) {
 
 // Do NOT change any of these structs.  They have to match the
 // structs  that the palm maps were made with.
@@ -181,7 +188,9 @@ void draw_palm_image_map(Widget w,
     long line_x, line_y;
     int vector;
     char status_text[MAX_FILENAME];
+    int draw_filled;
 
+    draw_filled = mdf->draw_filled;
 
     xastir_snprintf(filename, sizeof(filename), "%s/%s", dir, filenm);
 
@@ -275,7 +284,8 @@ void draw_palm_image_map(Widget w,
                 map_bottom, // Bottom
                 map_top,    // Top
                 map_left,   // Left
-                map_right); // Right
+                map_right,  // Right
+                1000);      // Default Map Level
 
             fclose(fn);
 
@@ -309,7 +319,16 @@ void draw_palm_image_map(Widget w,
             if (interrupt_drawing_now) {
                 fclose(fn);
                 // Update to screen
-                (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+                (void)XCopyArea(XtDisplay(da),
+                    pixmap,
+                    XtWindow(da),
+                    gc,
+                    0,
+                    0,
+                    (unsigned int)screen_width,
+                    (unsigned int)screen_height,
+                    0,
+                    0);
                 return;
             }
 
