@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tif.c,v 1.14 2004/10/11 19:15:26 tvrusso Exp $
+ * $Id: map_tif.c,v 1.33 2005/10/20 01:29:56 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
  */
 
 
+
 #include "config.h"
 #include "snprintf.h"
 
@@ -36,7 +37,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -79,9 +82,18 @@ extern int mag;
 #ifdef HAVE_LIBGEOTIFF
 //#include "cpl_csv.h"
 #include "xtiffio.h"
-#include "geotiffio.h"
+//#include "geotiffio.h"
 #include "geo_normalize.h"
-#include "projects.h"
+
+
+// Needed for GTIFProj4FromLatLong() replacement below
+#ifdef HAVE_LIBGDAL
+#  include "projects.h"
+#endif  // HAVE_LIBGDAL
+
+
+// Must be last include file
+#include "leak_detection.h"
 
 
 
@@ -172,6 +184,8 @@ void get_alt_fgd_path2(char *fullpath, int fullpath_length) {
 
 
 
+
+
 /***********************************************************
  * read_fgd_file()
  *
@@ -236,7 +250,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "WEST BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "West_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 25, " %f", f_west_bounding);
+                    if (1 != sscanf (ptr + 25, " %f", f_west_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"West Bounding:  %f\n",*f_west_bounding);
                     num_coordinates++;
@@ -248,7 +264,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "EAST BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "East_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 25, " %f", f_east_bounding);
+                    if (1 != sscanf (ptr + 25, " %f", f_east_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"East Bounding:  %f\n",*f_east_bounding);
                     num_coordinates++;
@@ -260,7 +278,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "NORTH BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "North_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 26, " %f", f_north_bounding);
+                    if (1 != sscanf (ptr + 26, " %f", f_north_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"North Bounding: %f\n",*f_north_bounding);
                     num_coordinates++;
@@ -272,7 +292,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "SOUTH BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "South_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 26, " %f", f_south_bounding);
+                    if (1 != sscanf (ptr + 26, " %f", f_south_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"South Bounding: %f\n",*f_south_bounding);
                     num_coordinates++;
@@ -316,6 +338,93 @@ int read_fgd_file ( char* tif_filename,
 }
 
 
+
+
+
+#ifndef HAVE_LIBGDAL
+#  define my_GTIFProj4FromLatLong GTIFProj4FromLatLong
+#else
+
+// If we have libgdal included then the GTIFProj4FromLatLong()
+// functions may not work.  In this case we use the function below
+// instead.
+
+
+// This next function was borrowed from the libgeotiff project.  We
+// have permission from Frank Warmerdam to use any code from
+// geotiff_proj4.c and relicense it under GPL, per private message
+// of 10/14/2005.  Frank wrote that module.
+//
+// When we used the original function from libgeotiff we'd lose topo
+// map capability if we linked in libgdal.  We avoid that problem by
+// having a local copy of the function here with a different name so
+// that libgdal can't try to override it.
+//
+//***********************************************************************
+//                        my_GTIFProj4FromLatLong()
+//
+//      Convert lat/long values to projected coordinate for a
+//      particular definition.
+//***********************************************************************
+
+int my_GTIFProj4FromLatLong( GTIFDefn * psDefn,
+                             int nPoints,
+                             double *padfX,
+                             double *padfY ) {
+
+    char    *pszProjection, **papszArgs;
+    PJ      *psPJ;
+    int     i;
+
+// --------------------------------------------------------------------
+//  Get a projection definition.
+// --------------------------------------------------------------------
+    pszProjection = GTIFGetProj4Defn( psDefn );
+
+    if( pszProjection == NULL )
+        return FALSE;
+
+// --------------------------------------------------------------------
+//      Parse into tokens for pj_init(), and initialize the projection
+// --------------------------------------------------------------------
+
+    papszArgs = CSLTokenizeStringComplex( pszProjection, " +", TRUE, FALSE );
+    free( pszProjection );
+
+    psPJ = pj_init( CSLCount(papszArgs), papszArgs );
+
+    CSLDestroy( papszArgs );
+
+    if( psPJ == NULL ) {
+        return FALSE;
+    }
+
+// --------------------------------------------------------------------
+//      Process each of the points.
+// --------------------------------------------------------------------
+
+    for( i = 0; i < nPoints; i++ ) {
+        projUV  sUV;
+
+        sUV.u = padfX[i] * DEG_TO_RAD;
+        sUV.v = padfY[i] * DEG_TO_RAD;
+
+        sUV = pj_fwd( sUV, psPJ );
+
+        padfX[i] = sUV.u;
+        padfY[i] = sUV.v;
+    }
+
+    pj_free( psPJ );
+
+    return TRUE;
+}
+#endif  // HAVE_LIBGDAL
+
+
+
+
+
 // Check the "GTIFProj4*" functions to see if we can use
 // datum.c instead for any of them.  These calls are all in
 // draw_geotiff_image_map().  It looks like they deal with
@@ -357,7 +466,7 @@ void draw_geotiff_image_map (Widget w,
 			     alert_entry *alert,
 			     u_char alert_color,
 			     int destination_pixmap,
-			     int draw_filled) {
+			     map_draw_flags *mdf) {
     char file[MAX_FILENAME];    /* Complete path/name of image file */
     char short_filenm[MAX_FILENAME];
     TIFF *tif = (TIFF *) 0;     /* Filehandle for tiff image file */
@@ -492,6 +601,10 @@ void draw_geotiff_image_map (Widget w,
     unsigned long view_top_minus_pixel_height;
     int proj_is_latlong;
     short PCS;
+    int usgs_drg;
+    char *imagedesc;
+
+    usgs_drg = mdf->usgs_drg;              // yes, no, or auto
 
     if (debug_level & 16)
         fprintf(stderr,"%s/%s\n", dir, filenm);
@@ -688,18 +801,20 @@ void draw_geotiff_image_map (Widget w,
      * or keys that I need individually instead.  I
      * need "defn" for the GTIFProj4ToLatLong calls though.
      */
-    if (GTIFGetDefn (gtif, &defn))
-    {
+    if (GTIFGetDefn (gtif, &defn)) {
         if (debug_level & 16)
             GTIFPrintDefn (&defn, stdout);
     }
+    else {
+        fprintf(stderr,"GTIFGetDefn failed\n");
+    }
 
     proj_is_latlong=FALSE;
-    if( !GTIFKeyGet(gtif,ProjectedCSTypeGeoKey, &PCS,0,1))
-      {
-	fprintf(stderr,"Warning: no PCS in geotiff file %s, assuming map is in lat/lon!\n", filenm);
-	proj_is_latlong=TRUE;
-      }
+
+    if( !GTIFKeyGet(gtif,ProjectedCSTypeGeoKey, &PCS,0,1)) {
+          //fprintf(stderr,"Warning: no PCS in geotiff file %s, assuming map is in lat/lon!\n", filenm);
+        proj_is_latlong=TRUE;
+    }
 
     /* Fetch a few TIFF fields for this image */
     if ( !TIFFGetField (tif, TIFFTAG_IMAGEWIDTH, &width) ) {
@@ -712,6 +827,23 @@ void draw_geotiff_image_map (Widget w,
         fprintf(stderr,"No height tag found in file, setting it to 6840\n");
     }
 
+    // If we're autodetecting usgs_drg, check the image description tag
+    // Note, the TIFFGetField doesn't allocate a string, it returns a pointer
+    // to an existing one.  Don't free it!
+    if (usgs_drg == 2) {
+        if ( TIFFGetField (tif, TIFFTAG_IMAGEDESCRIPTION, &imagedesc)) {
+            if (strncasecmp(imagedesc,"USGS GeoTIFF DRG",16) == 0) {
+                usgs_drg = 1;   // Yes
+            } 
+            else {
+                usgs_drg = 0;  // No
+            }
+        }
+        else {
+                usgs_drg = 0;  // No tag, assume not a usgs topo 
+        }
+            
+    }
 
     /*
      * If we don't have an associated .fgd file for this map,
@@ -755,6 +887,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if ( proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )   // Do all 4 of these in one call?
         {
             if (debug_level & 16) {
@@ -763,6 +896,10 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            fprintf(stderr,"Failed GTIFProj4LatLong() call\n");
+        }
+
         f_NW_x_bounding = (float)xxx;
         f_NW_y_bounding = (float)yyy;
 
@@ -777,6 +914,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -785,6 +923,10 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            fprintf(stderr,"Failed GTIFProj4LatLong() call\n");
+        }
+
         f_NE_x_bounding = (float)xxx;
         f_NE_y_bounding = (float)yyy;
 
@@ -798,6 +940,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -806,6 +949,10 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            fprintf(stderr,"Failed GTIFProj4LatLong() call\n");
+        }
+
         f_SW_x_bounding = (float)xxx;
         f_SW_y_bounding = (float)yyy;
 
@@ -819,6 +966,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -827,6 +975,10 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            fprintf(stderr,"Failed GTIFProj4LatLong() call\n");
+        }
+
         f_SE_x_bounding = (float)xxx;
         f_SE_y_bounding = (float)yyy;
 
@@ -1064,7 +1216,8 @@ void draw_geotiff_image_map (Widget w,
             south_bounding_wgs84,   // Bottom
             north_bounding_wgs84,   // Top
             west_bounding_wgs84,    // Left
-            east_bounding_wgs84);   // Right
+            east_bounding_wgs84,    // Right
+            0);                     // Default Map Level
 
         //Free any memory used and return
         /* We're finished with the geoTIFF key parser, so get rid of it */
@@ -1117,7 +1270,16 @@ void draw_geotiff_image_map (Widget w,
         GTIFFree (gtif);
         XTIFFClose (tif);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1226,17 +1388,37 @@ Samples Per Pixel: 1
 
 
     /* Print out the colormap info */
-    //    if (debug_level & 16) {
-    //        int l;
+    //if (debug_level & 16) {
+    //    int l;
     //
-    //        for (l = 0; l < num_colors; l++)
-    //            fprintf(stderr,"   %5u: %5u %5u %5u\n",
-    //                        l,
-    //                        red_orig[l],
-    //                        green_orig[l],
-    //                        blue_orig[l]);
-    //    }
-
+    //    for (l = 0; l < num_colors; l++)
+    //    fprintf(stderr,"   %5u: %5u %5u %5u\n",
+    //    l,
+    //    red_orig[l],
+    //    green_orig[l],
+    //    blue_orig[l]);
+    //}
+    // Example output from a USGS 7.5' map:
+    //
+    //   0:     0     0     0   black
+    //   1: 65280 65280 65280   light grey
+    //   2:     0 38656 41984
+    //   3: 51968     0  5888
+    //   4: 33536 16896  9472   contour lines, brownish-red
+    //   5: 51456 59904 40192   green
+    //   6: 35072 13056 32768   purple?  freeways, some roads
+    //   7: 65280 59904     0
+    //   8: 42752 57856 57856   blue, bodies of water
+    //   9: 65280 47104 47104   red brick color, cities?
+    //  10: 55808 45824 54784   purple.  freeways, some roads
+    //  11: 53504 53504 53504   grey
+    //  12: 52992 41984 36352   contour lines, tan, more dots/less lines
+    //  13:     0     0     0   black, unused slot?
+    //  14:     0     0     0   black, unused slot?
+    //  15:     0     0     0   black, unused slot?
+    //  16:     0     0     0   black, unused slot?
+    // The rest are all 0's.
+    
 
     if (crop_it)    // USGS geoTIFF map
     {
@@ -1262,7 +1444,16 @@ Samples Per Pixel: 1
             GTIFFree (gtif);
             XTIFFClose (tif);
             // Update to screen
-            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
             return;
         }
 
@@ -1275,14 +1466,14 @@ Samples Per Pixel: 1
         yyy = (double)f_NW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )     // Do all 4 in one call?
-        {
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) ) {     // Do all 4 in one call?
+
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
 
             /* Convert from PCS coordinates to image pixel coordinates */
-            if ( GTIFPCSToImage( gtif, &xxx, &yyy ) )           // Do all 4 in one call?
-            {
+            if ( GTIFPCSToImage( gtif, &xxx, &yyy ) ) { // Do all 4 in one call?
+
                 if (debug_level & 16)
                     fprintf(stderr,"X/Y Pixels: %f, %f\n", xxx, yyy);
 
@@ -1353,7 +1544,7 @@ Samples Per Pixel: 1
         yyy = (double)f_NE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1431,7 +1622,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1509,7 +1700,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1702,11 +1893,14 @@ right_crop = width - 1;
                 my_colors[l].green = (uint16)(green_orig[l] * raster_map_intensity);
                 my_colors[l].blue  =  (uint16)(blue_orig[l] * raster_map_intensity);
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         case PHOTOMETRIC_MINISBLACK:
@@ -1716,11 +1910,14 @@ right_crop = width - 1;
                 my_colors[l].red = my_colors[l].green = my_colors[l].blue =
                     (uint16)(v * raster_map_intensity) << 8;
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         case PHOTOMETRIC_MINISWHITE:
@@ -1730,11 +1927,14 @@ right_crop = width - 1;
                 my_colors[l].red = my_colors[l].green = my_colors[l].blue =
                   (uint16)(v * raster_map_intensity) << 8;
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         }
@@ -2224,8 +2424,22 @@ right_crop = width - 1;
     // probably).  A higher number means less rows skipped,
     // which improves the look but slows the map drawing down.
     //
-    if (have_PixelScale && !proj_is_latlong) {
-        SkipRows = (int)( ( scale_y / ( *PixelScale * 3.15 ) ) + 0.5 );
+    // This needs modification somewhat when the raster is already in 
+    // lat/long, and the PixelScale is already in degrees per pixel.
+    // The scale_y is the integer number of hundredths of seconds per pixel.
+    // The decision of how many rows to skip can simply be made by converting
+    // the PixelScale to the same units as scale_y, and skipping accordingly.
+
+    if (have_PixelScale) {
+        double coef;
+        if (!proj_is_latlong) {
+            coef = 3.15;
+        } 
+        else {
+            coef=100*60*60;  // xastir coords are in 1/100 of a second,
+                             // and lat/lon pixel scales will be in degrees
+        }
+        SkipRows = (int)( ( scale_y / ( *PixelScale * coef ) ) + 0.5 );
         if (SkipRows < 1)
             SkipRows = 1;
         if (SkipRows > (int)(height / 10) )
@@ -2285,6 +2499,22 @@ right_crop = width - 1;
     }
 
 
+    // Set the bit-blitting function to XOR.  This is only used if
+    // DRG_XOR_colors is set to a non-zero value, but we set it here
+    // outside the pixel loop for speed reasons.
+    //
+    if (DRG_XOR_colors) {
+        (void)XSetLineAttributes (XtDisplay (w), gc_tint, 1, LineSolid, CapButt,JoinMiter);
+
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x27]);  // yellow
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x0f]); // White
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x03]); // cyan
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x06]); // orange
+ 
+        (void)XSetFunction (XtDisplay (da), gc_tint, GXxor);
+    }
+
+
     // Iterate over the rows of interest only.  Using the rectangular
     // top/bottom crop values for these is ok at this point.
     //
@@ -2483,11 +2713,64 @@ if (current_right >= width)
                     sxx = (xastir_current_x - x_long_offset) / scale_x;
                     syy = (xastir_total_y   - y_lat_offset ) / scale_y;
 
-                    // Set the color for the pixel
-                    XSetForeground (XtDisplay (w), gc, my_colors[*(imageMemory + column)].pixel);
 
-                    // And draw the pixel
-                    XFillRectangle (XtDisplay (w), pixmap, gc, sxx, syy, stepwc, stephc);
+
+// If we wish some colors to be transparent, we could check for a
+// color match here and refuse to draw those that are on our list of
+// transparent colors.  Might be useful for drawing say contour
+// lines on top of satellite images.  Since the USGS topo's use the
+// same colormap for all of the topo's (I believe that is true) then
+// we could just compare on the index into the array instead of the
+// color contents in the array.
+//
+    // Example output from a USGS 7.5' map:
+    //
+    //   0:     0     0     0   black
+    //   1: 65280 65280 65280   light grey
+    //   2:     0 38656 41984
+    //   3: 51968     0  5888
+    //   4: 33536 16896  9472   contour lines, brownish-red
+    //   5: 51456 59904 40192   green
+    //   6: 35072 13056 32768   purple?  freeways, some roads
+    //   7: 65280 59904     0
+    //   8: 42752 57856 57856   blue, bodies of water
+    //   9: 65280 47104 47104   red brick color, cities?
+    //  10: 55808 45824 54784   purple.  freeways, some roads
+    //  11: 53504 53504 53504   grey
+    //  12: 52992 41984 36352   contour lines, tan, more dots/less lines
+    //  13:     0     0     0   black, unused slot?
+    //  14:     0     0     0   black, unused slot?
+    //  15:     0     0     0   black, unused slot?
+    //  16:     0     0     0   black, unused slot?
+    // The rest are all 0's.
+                    // either show all colors (usgs_drg==0) or show selected
+                    // colors (usgs_drg == 1) 
+                    if ( usgs_drg == 0 ||
+                         (usgs_drg ==1 && (*(imageMemory + column) >= 13
+                          || DRG_show_colors[*(imageMemory + column)] == 1))) {
+
+
+                        // If this is set, use gc_tint for drawing
+                        // with "GXxor" as the bit-blit operation.
+                        //
+                        if (DRG_XOR_colors) {
+
+                            // Draw the pixel using "gc_tint"
+                            // instead of "gc".
+
+                            XSetForeground (XtDisplay (w), gc_tint, my_colors[*(imageMemory + column)].pixel);
+ 
+                            XFillRectangle (XtDisplay (w), pixmap, gc_tint, sxx, syy, stepwc, stephc);
+                        }
+                        else {
+
+                            // Set the color for the pixel
+                            XSetForeground (XtDisplay (w), gc, my_colors[*(imageMemory + column)].pixel);
+
+                            // And draw the pixel
+                            XFillRectangle (XtDisplay (w), pixmap, gc, sxx, syy, stepwc, stephc);
+                        }
+                    }
                 }
             }
         }
