@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: maps.h,v 1.46 2004/03/04 23:12:34 we7u Exp $
+ * $Id: maps.h,v 1.56 2005/09/26 02:35:38 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -76,8 +76,16 @@ typedef struct _map_index_record{
     int accessed;
     int max_zoom;       // Specify maximum zoom at which this layer is drawn.
     int min_zoom;       // Specify minimum zoom at which this layer is drawn.
-    int map_layer;      // For future expansion.  Specify which layer to draw the map on.
-    int draw_filled;    // For future expansion.  Specify whether to fill polygons when drawing.
+    int map_layer;      // Specify which layer to draw the map on.
+    int draw_filled;    // Specify whether to fill polygons when drawing.
+                        // 0 = Global No-Fill (Vector)
+                        // 1 = Global Fill
+                        // 2 = Auto (dbfawk controls it if present)
+    int usgs_drg;       // Specify whether the map has USGS DRG colormap
+                        // and should have color configuration applied
+                        // 0 = No
+                        // 1 = Yes
+                        // 2 = Auto (detect from TIFFTAG_IMAGEDESCRIPTION)
     int selected;       // Specifies if map is currently selected
     int temp_select;    // Temporary selection used in map properties dialog
     int auto_maps;      // Specifies if map included in automaps function
@@ -92,6 +100,8 @@ typedef struct {
     unsigned long y_lat;
 } tiepoint;
 
+extern void recompute_lat_long(void);
+
 void draw_point(Widget w,
                  unsigned long x1,
                  unsigned long y1,
@@ -126,17 +136,21 @@ void load_auto_maps(Widget w, char *dir);
 void load_maps(Widget w);
 void fill_in_new_alert_entries(Widget w, char *dir);
 void load_alert_maps(Widget w, char *dir);
-void  index_update_xastir(char *filename, unsigned long bottom, unsigned long top, unsigned long left, unsigned long right);
-void  index_update_ll(char *filename, double bottom, double top, double left, double right);
+void  index_update_xastir(char *filename, unsigned long bottom, unsigned long top, unsigned long left, unsigned long right, int default_map_layer);
+void  index_update_ll(char *filename, double bottom, double top, double left, double right, int default_map_layer);
 void draw_grid (Widget w);
 void Snapshot(void);
 extern int index_retrieve(char *filename, unsigned long *bottom,
     unsigned long *top, unsigned long *left, unsigned long *right,
     int *max_zoom, int *min_zoom, int *map_layer, int *draw_filled, 
-    int *automaps);
+    int *usgs_drg, int *automaps);
 extern void index_restore_from_file(void);
 extern void index_save_to_file(void);
 extern void map_indexer(int parameter);
+extern void get_viewport_lat_lon(double *xmin, 
+                                 double *ymin, 
+                                 double *xmax,
+                                 double *ymax);
 extern int map_visible (unsigned long bottom_map_boundary,
                         unsigned long top_map_boundary,
                         unsigned long left_map_boundary,
@@ -146,6 +160,10 @@ extern int map_visible_lat_lon (double f_bottom_map_boundary,
                                 double f_left_map_boundary,
                                 double f_right_map_boundary,
                                 char *error_message);
+extern int map_inside_viewport_lat_lon(double map_min_y,
+                                       double map_max_y,
+                                       double map_min_x,
+                                       double map_max_x);
 extern void draw_label_text (Widget w, int x, int y, int label_length, int color, char *label_text);
 extern void draw_rotated_label_text (Widget w, int rotation, int x, int y, int label_length, int color, char *label_text, int fontsize);
 extern void draw_centered_label_text (Widget w, int rotation, int x, int y, int label_length, int color, char *label_text, int fontsize);
@@ -157,16 +175,14 @@ extern int print_auto_rotation;
 extern int print_auto_scale;
 extern int print_in_monochrome;
 extern int print_invert;
-extern int  locate_place(Widget w, char *name, char *state, char *county, char *quad, char* type, char *filename, int follow_case, int get_match);
+
+extern int  gnis_locate_place(Widget w, char *name, char *state,
+char *county, char *quad, char* type, char *filename, int
+follow_case, int get_match, char match_array_name[50][200], long
+match_array_lat[50], long match_array_long[50]);
+
+
 extern void maps_init(void);
-extern int convert_from_xastir_coordinates ( float *f_longitude,
-                                      float *f_latitude,
-                                      long x,
-                                      long y );
-extern int convert_to_xastir_coordinates (unsigned long* x,
-					  unsigned long* y,
-					  float f_longitude,
-					  float f_latitude);
 enum map_onscreen_enum {MAP_NOT_VIS=0,MAP_IS_VIS,MAP_NOT_INDEXED};
 extern enum map_onscreen_enum map_onscreen(long left, long right, long top, long bottom, int checkpercentage);
 extern enum map_onscreen_enum map_onscreen_index(char *filename);
@@ -188,6 +204,16 @@ extern void map_plot (Widget w, long max_x, long max_y, long x_long_cord, long y
 
 extern void map_gdal_init(void);
 
+extern int check_trans (XColor c, unsigned long c_trans_color);
+
+// A struct to pass down in to map driver functions so they can have 
+// driver-specific flags.  Most drivers won't care about any (or even all)
+// of the flags, but this way we can just pass a single pointer rather than
+// adding new arguments to the generic interface each time we want new flags
+typedef struct {
+    int draw_filled;
+    int usgs_drg;
+} map_draw_flags;
 
 #endif /* __XASTIR_MAPS_H */
 
