@@ -1,9 +1,9 @@
 /*
- * $Id: popup_gui.c,v 1.17 2004/09/17 19:46:06 we7u Exp $
+ * $Id: popup_gui.c,v 1.21 2005/10/02 06:26:49 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,10 @@
 #include "rotated.h"
 #include "snprintf.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 static Popup_Window pw[MAX_POPUPS];
 static Popup_Window pwb;
@@ -154,11 +158,12 @@ void popup_message_always(char *banner, char *message) {
 
 begin_critical_section(&popup_message_dialog_lock, "popup_gui.c:popup_message" );
 
-            pw[i].popup_message_dialog = XtVaCreatePopupShell(banner,xmDialogShellWidgetClass,Global.top,
-                                  XmNdeleteResponse,XmDESTROY,
-                                  XmNdefaultPosition, FALSE,
-                                  XmNtitleString,banner,
-                                  NULL);
+            pw[i].popup_message_dialog = XtVaCreatePopupShell(banner,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
+                XmNdefaultPosition, FALSE,
+                XmNtitleString,banner,
+                NULL);
 
             pw[i].pane = XtVaCreateWidget("popup_message pane",xmPanedWindowWidgetClass, pw[i].popup_message_dialog,
                           XmNbackground, colors[0xff],
@@ -287,7 +292,13 @@ void popup_ID_message(char *banner, char *message) {
         // be seen.
         (void)XSetForeground(XtDisplay(da),gc,MY_BG_COLOR); // Not a mistake!
         (void)XSetBackground(XtDisplay(da),gc,MY_BG_COLOR);
-        (void)XFillRectangle(XtDisplay(appshell),pixmap_alerts,gc,0,0,screen_width,screen_height);
+        (void)XFillRectangle(XtDisplay(appshell),
+            pixmap_alerts,
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height);
 
         /* load font */
         if(!id_font) {
@@ -335,7 +346,16 @@ void popup_ID_message(char *banner, char *message) {
 
         // Write it to the screen.  Symbols/tracks will disappear during
         // this short interval time.
-        (void)XCopyArea(XtDisplay(da),pixmap_alerts,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap_alerts,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
     }
     else {  // ATV Screen ID is not enabled
         pending_ID_message = 0;
