@@ -1,10 +1,10 @@
 /*
  * snprintf.c - a portable implementation of snprintf and vsnprintf
  *
- * $Id: snprintf.c,v 1.4 2003/02/20 09:51:41 we7u Exp $
+ * $Id: snprintf.c,v 1.7 2005/08/17 19:12:05 we7u Exp $
  */
 /*
- * Portions Copyright (C) 2000-2003 Xastir Development Team
+ * Portions Copyright (C) 2000-2005 The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 #include "snprintf.h"
 #include "config.h"
 
+
 /*
  * Copyright Patrick Powell 1995
  * This code is based on code written by Patrick Powell (papowell@astart.com)
@@ -110,6 +111,12 @@
 #  define VA_END va_end(ap)
 #endif  // __STDC__
 
+
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 #ifdef HAVE_LONG_DOUBLE
 #define LDOUBLE long double
 #else   // HAVE_LONG_DOUBLE
