@@ -1,9 +1,9 @@
 /*
- * $Id: view_message_gui.c,v 1.23 2004/10/18 18:58:10 we7u Exp $
+ * $Id: view_message_gui.c,v 1.31 2005/10/02 06:26:49 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,7 +35,11 @@
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <strings.h>
+#ifdef HAVE_STRING_H
+#  include <string.h>
+#else
+#  include <strings.h>
+#endif
 #include <ctype.h>
 #include <sys/types.h>
 
@@ -54,9 +58,10 @@
 #include "main.h"
 #include "util.h"
 
-#ifdef HAVE_DMALLOC
-#include <dmalloc.h>
-#endif  // HAVE_DMALLOC
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 Widget All_messages_dialog = NULL;
 Widget view_messages_text = NULL;
@@ -65,7 +70,7 @@ Widget vm_dist_data = NULL;
 static xastir_mutex All_messages_dialog_lock;
 
 int vm_range;
-int view_message_limit;
+int view_message_limit = 10000;
 
 
 
@@ -97,14 +102,18 @@ void view_message_print_record(Message *m_fill) {
         if ((temp = malloc((size_t)my_size)) == NULL)
             return;
 
-        sprintf(temp,"%-9s>%-9s seq:%5s type:%c :%s\n",
+        sprintf(temp,"%-9s>%-9s %s:%5s %s:%c :%s\n",
             m_fill->from_call_sign,
             m_fill->call_sign,
+            langcode("WPUPMSB013"),
             m_fill->seq,
+            langcode("WPUPMSB014"),
             m_fill->type,
             m_fill->message_line);
 
         pos = (int)XmTextGetLastPosition(view_messages_text);
+
+
         XmTextInsert(view_messages_text, pos, temp);
         pos += strlen(temp);
         while (pos > view_message_limit) {
@@ -162,7 +171,7 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
             data1[95]='\0';
             xastir_snprintf(data2,
                 sizeof(data2),
-                "%s",
+                "\n\t%s",
                 message+95);
         } else {
             xastir_snprintf(data1,
@@ -175,10 +184,10 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
         if (strncmp(call_sign, "java",4) == 0) {
             xastir_snprintf(call_sign,
                 MAX_CALLSIGN+1,
-                "Broadcast");
+                langcode("WPUPMSB015") );   // Broadcast
             xastir_snprintf(temp,
                 my_size,
-                "%s %s\n%s\n%s\n",
+                "%s %s\t%s%s\n",
                 from_call,
                 call_sign,
                 data1,
@@ -186,10 +195,10 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
         } else if (strncmp(call_sign, "USER", 4) == 0) {
             xastir_snprintf(call_sign,
                 MAX_CALLSIGN+1,
-                "Broadcast");
+                langcode("WPUPMSB015") );   // Broadcast
             xastir_snprintf(temp,
                 my_size,
-                "%s %s\n%s\n%s\n",
+                "%s %s\t%s%s\n",
                 from_call,
                 call_sign,
                 data1,
@@ -197,7 +206,7 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
         } else
             xastir_snprintf(temp,
                 my_size,
-                "%s to %s via:%c\n%s\n%s\n",
+                "%s to %s via:%c\t%s%s\n",
                 from_call,
                 call_sign,
                 from,
@@ -290,9 +299,8 @@ void view_all_messages( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData
 begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_messages" );
 
         All_messages_dialog = XtVaCreatePopupShell(langcode("AMTMW00001"),
-                xmDialogShellWidgetClass,
-                Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
 
@@ -465,4 +473,3 @@ end_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_mes
     }
 }
 
-
