@@ -1,9 +1,9 @@
 /*
- * $Id: wx_gui.c,v 1.35 2004/10/18 18:58:10 we7u Exp $
+ * $Id: wx_gui.c,v 1.43 2005/10/20 20:12:59 gstueve Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,6 +55,9 @@
 #endif  // HAVE_XBAE_MATRIX_H
 #include <X11/cursorfont.h>
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 /************ Weather Alerts ****************/
@@ -106,7 +109,7 @@ end_critical_section(&wx_detailed_alert_shell_lock, "wx_gui.c:wx_detailed_alert_
 void wx_alert_finger_output( Widget widget, char *handle) {
     static Widget pane, my_form, mess, button_cancel,wx_detailed_alert_list;
     Atom delw;
-    Arg al[20];                     // Arg List
+    Arg al[50];                     // Arg List
     register unsigned int ac = 0;   // Arg Count
     char temp[1024];
     XmString item;
@@ -121,11 +124,12 @@ void wx_alert_finger_output( Widget widget, char *handle) {
 
 begin_critical_section(&wx_detailed_alert_shell_lock, "wx_gui.c:wx_alert_double_click_action" );
 
-        wx_detailed_alert_shell = XtVaCreatePopupShell(langcode("WPUPWXA001"), xmDialogShellWidgetClass, Global.top,
-                        XmNdeleteResponse, XmDESTROY,
-                        XmNdefaultPosition, FALSE,
-                        XmNminWidth, 600,
-                        NULL);
+        wx_detailed_alert_shell = XtVaCreatePopupShell(langcode("WPUPWXA001"),
+            xmDialogShellWidgetClass, appshell,
+            XmNdeleteResponse, XmDESTROY,
+            XmNdefaultPosition, FALSE,
+            XmNminWidth, 600,
+            NULL);
 
         pane = XtVaCreateWidget("wx_alert_double_click_action pane",xmPanedWindowWidgetClass, wx_detailed_alert_shell,
                         XmNbackground, colors[0xff],
@@ -310,7 +314,44 @@ end_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_destroy_shell" );
 }
 
 
+static int alert_comp(const void *a, const void *b) {
+    alert_entry *a_entry = *(alert_entry **)a;
+    alert_entry *b_entry = *(alert_entry **)b;
+    int a_active, b_active;
 
+    if (a_entry->title[on_screen] && !b_entry->title[on_screen])
+        return (-1);
+
+    if (!a_entry->title[0] && b_entry->title[0])
+        return (1);
+
+    if (a_entry->flags[on_screen] == 'Y' && b_entry->flags[on_screen] != 'Y')
+        return (-1);
+
+    if (a_entry->flags[on_screen] != 'Y' && b_entry->flags[on_screen] == 'Y')
+        return (1);
+
+    if (a_entry->flags[on_screen] == '?' && b_entry->flags[on_screen] == 'N')
+        return (-1);
+
+    if (a_entry->flags[on_screen] == 'N' && b_entry->flags[on_screen] == '?')
+        return (1);
+
+    a_active = alert_active(a_entry, ALERT_ALL);
+    b_active = alert_active(b_entry, ALERT_ALL);
+    if (a_active && b_active) {
+        if (a_active - b_active) {
+            return (a_active - b_active);
+        }
+    } else if (a_active) {
+        return (-1);
+    }
+    else if (b_active) {
+        return (1);
+    }
+
+    return (strcmp(a_entry->title, b_entry->title));
+}
 
 
 void wx_alert_update_list(void) {
@@ -319,59 +360,79 @@ void wx_alert_update_list(void) {
     int max_item_count; // max dialog lines
     char temp[600];
     XmString item;
+    static alert_entry **alert_list;
+    static int alert_list_limit;
 
     if (wx_alert_shell) {
+        struct hashtable_itr *iterator;
+        alert_entry *alert;
 
 begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 
         // Get the previous alert count from the alert list window
         XtVaGetValues(wx_alert_list, XmNitemCount, &max_item_count, NULL);
 
-        ii = 0; // Current dialog count
+	if ((nn = alert_list_count()) > alert_list_limit) {
+	    alert_entry **tmp = realloc(alert_list, nn * sizeof(alert_entry *));
+	    if (tmp) {
+		alert_list = tmp;
+		alert_list_limit = nn;
+	    } else {
+		fprintf(stderr, "wx_gui: Alert list allocation error\n");
+		exit(1);
+	    }
 
-        // Step through the alert list.  Create a string for each
+	}
+        // Iterate through the alert hash.  Create a string for each
         // non-expired/non-blank entry.
-        for (nn = 0; nn < alert_max_count; nn++) {
-            char status[10];
-
-            // Check whether alert slot is empty/filled
-            if (alert_list[nn].title[0] == '\0')    // It's empty
-                continue;
+        iterator = create_wx_alert_iterator();
+        for (nn = 0, alert = get_next_wx_alert(iterator); iterator != NULL && alert; alert = get_next_wx_alert(iterator)) {
 
+            // Check whether alert record is empty/filled.  This
+            // code is from the earlier array implementation.  If
+            // we're expiring records from our hash properly we
+            // probably don't need this anymore.
+            //
+            if (alert->title[0] == '\0') {    // It's empty
+		fprintf(stderr, "wx_gui:alert->title NULL\n");
+                break;
+            }
+	    alert_list[nn++] = alert;
+	}
+	qsort(alert_list, nn, sizeof(alert_entry *), alert_comp);
+	for (ii = 0; ii < nn; 
+) {
+            alert = alert_list[ii];
             // AFGNPW      NWS-WARN    Until: 191500z   AK_Z213   WIND               P7IAA
             // TSATOR      NWS-ADVIS   Until: 190315z   OK_C127   TORNDO             H2VAA
             //xastir_snprintf(temp, sizeof(temp), "%-9s   %-9s   Until: %-7s   %-7s   %-20s   %s",
 
-            xastir_snprintf(status,
-                sizeof(status),
-                "   ");
-
             xastir_snprintf(temp,
                 sizeof(temp),
-                "%-9s%s   %-9s   %c%c @%c%c%c%cz ==> %c%c @%c%c%c%cz %s %-7s   %s   %s%s%s%s",
-                alert_list[nn].from,
-                alert_list[nn].seq,
-                alert_list[nn].to,
-                alert_list[nn].issue_date_time[0],
-                alert_list[nn].issue_date_time[1],
-                alert_list[nn].issue_date_time[2],
-                alert_list[nn].issue_date_time[3],
-                alert_list[nn].issue_date_time[4],
-                alert_list[nn].issue_date_time[5],
-                alert_list[nn].activity[0],
-                alert_list[nn].activity[1],
-                alert_list[nn].activity[2],
-                alert_list[nn].activity[3],
-                alert_list[nn].activity[4],
-                alert_list[nn].activity[5],
-                status,
-                alert_list[nn].title,
-                alert_list[nn].alert_tag,
-                alert_list[nn].desc0,
-                alert_list[nn].desc1,
-                alert_list[nn].desc2,
-                alert_list[nn].desc3);
-
+                "%-9s %-5s %-9s %c%c @%c%c%c%cz ==> %c%c @%c%c%c%cz %c%c %-7s %s %s%s%s%s",
+                alert->from,
+                alert->seq,
+                alert->to,
+                alert->issue_date_time[0],
+                alert->issue_date_time[1],
+                alert->issue_date_time[2],
+                alert->issue_date_time[3],
+                alert->issue_date_time[4],
+                alert->issue_date_time[5],
+                alert->activity[0],
+                alert->activity[1],
+                alert->activity[2],
+                alert->activity[3],
+                alert->activity[4],
+                alert->activity[5],
+                alert->flags[on_screen],
+		alert->flags[source],
+                alert->title,
+                alert->alert_tag,
+                alert->desc0,
+                alert->desc1,
+                alert->desc2,
+                alert->desc3);
 
             item = XmStringCreateLtoR(temp, XmFONTLIST_DEFAULT_TAG);
 
@@ -382,15 +443,18 @@ begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
             // window yet.  Add it.
             if (max_item_count < ii) {
                 XmListAddItemUnselected(wx_alert_list, item, 0);
-            }
-            else {  // Replace it in the window.  Note: This might
-                    // re-order the list each time.
+            } else {
+		// Replace it in the window.  Note: This might re-order the list each time.
                 XmListReplaceItemsPosUnselected(wx_alert_list, &item, 1, ii);
             }
 
             XmStringFree(item);
 
         }   // End of for loop
+#ifndef USING_LIBGC
+//fprintf(stderr,"free iterator 9\n");
+        if (iterator) free(iterator);
+#endif  // USING_LIBGC
 
         // If we have fewer alerts now, delete the extras from the window
         if (ii < max_item_count) {
@@ -409,18 +473,19 @@ end_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 void Display_Wx_Alert( /*@unused@*/ Widget wdgt, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     static Widget pane, my_form, mess, button_cancel;
     Atom delw;
-    Arg al[20];                    /* Arg List */
+    Arg al[50];                    /* Arg List */
     register unsigned int ac = 0;           /* Arg Count */
 
     if(!wx_alert_shell) {
 
 begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:Display_Wx_Alert" );
 
-        wx_alert_shell = XtVaCreatePopupShell(langcode("WPUPWXA001"), xmDialogShellWidgetClass, Global.top,
-                        XmNdeleteResponse, XmDESTROY,
-                        XmNdefaultPosition, FALSE,
-                        XmNminWidth, 600,
-                        NULL);
+        wx_alert_shell = XtVaCreatePopupShell(langcode("WPUPWXA001"),
+            xmDialogShellWidgetClass, appshell,
+            XmNdeleteResponse, XmDESTROY,
+            XmNdefaultPosition, FALSE,
+            XmNminWidth, 600,
+            NULL);
 
         pane = XtVaCreateWidget("Display_Wx_Alert pane",xmPanedWindowWidgetClass, wx_alert_shell,
                         XmNbackground, colors[0xff],
@@ -619,10 +684,11 @@ void WX_station( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@un
 
 begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:WX_station" );
 
-        wx_station_dialog = XtVaCreatePopupShell(langcode("WXPUPSI000"),xmDialogShellWidgetClass,Global.top,
-                                  XmNdeleteResponse,XmDESTROY,
-                                  XmNdefaultPosition, FALSE,
-                                  NULL);
+        wx_station_dialog = XtVaCreatePopupShell(langcode("WXPUPSI000"),
+            xmDialogShellWidgetClass, appshell,
+            XmNdeleteResponse, XmDESTROY,
+            XmNdefaultPosition, FALSE,
+            NULL);
 
         pane = XtVaCreateWidget("WX_station pane",xmPanedWindowWidgetClass, wx_station_dialog,
                           XmNbackground, colors[0xff],
