@@ -1,8 +1,8 @@
 /*
- * $Id: x_spider.c,v 1.28 2004/10/03 05:43:57 we7u Exp $
+ * $Id: x_spider.c,v 1.46 2005/10/26 19:11:37 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2004  The Xastir Group
+ * Copyright (C) 2003-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -117,7 +117,7 @@
 #include <signal.h>
 #include <termios.h>
 #include <pwd.h>
-#include <pthread.h>
+
 #include <termios.h>
 #include <setjmp.h>
 #include <sys/socket.h>
@@ -166,10 +166,26 @@
 #define SIGRET  void
 #endif  // SIGRET
 
+// Must be last include file
+#include "leak_detection.h"
 
-// This is from util.h/util.c.  We can't include util.h here because
-// it causes other problems.
+
+
+// Define this if you wish to use this as a standalone program
+// instead of as a function in another program.
+//
+//#define STANDALONE_PROGRAM
+
+
+
+// These are from util.h/util.c.  We can't include util.h here
+// because it causes other problems.
 extern short checkHash(char *theCall, short theHash);
+extern void get_timestamp(char *timestring);
+extern void split_string( char *data, char *cptr[], int max );
+
+// From database.h
+extern char my_callsign[];
 
 
 typedef struct _pipe_object {
@@ -184,10 +200,16 @@ typedef struct _pipe_object {
 pipe_object *pipe_head = NULL;
 //int master_fd = -1; // Start with an invalid value
 
-pipe_object *xastir_pipe = NULL;
+pipe_object *xastir_tcp_pipe = NULL;
+pipe_object *xastir_udp_pipe = NULL;
 
-int pipe_xastir_to_server = -1;
-int pipe_server_to_xastir = -1;
+// TCP server pipes to/from Xastir proper
+int pipe_xastir_to_tcp_server = -1;
+int pipe_tcp_server_to_xastir = -1;
+
+// UDP server pipes to/from Xastir proper
+int pipe_xastir_to_udp_server = -1; // (not currently used)
+int pipe_udp_server_to_xastir = -1;
 
 
 
@@ -345,12 +367,19 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
     }
 
 
+    //Send our callsign to spider clients as "#callsign" much like APRS-IS sends "# javaAPRS"
+    // # xastir 1.5.1 callsign:<mycall>
+    sprintf(line,"# Welcome to Xastir's server port, callsign: %s\r\n",my_callsign);
+    writen(sockfd,line,strlen(line));
+
     // Infinite loop
     for ( ; ; ) {
 
         //
         // Read data from socket, write to pipe (parent)
         //
+        if (!sockfd)    // Socket is closed
+            return; // Connection terminated
  
         n = readline(sockfd, line, MAXLINE);
         if (n == 0) {
@@ -363,13 +392,17 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
                 //fprintf(stderr,"EAGAIN or EWOULDBLOCK\n");
             }
             else {  // Non-normal error.  Report it.
-                fprintf(stderr,"str_echo2: Readline error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Readline error socket: %d\n",errno);
+//close(sockfd);
+return;
             }
         }
         else {  // We received some data.  Send it down the pipe.
 //            fprintf(stderr,"str_echo2: %s\n",line);
             if (writen(pipe_to_parent, line, n) != n) {
-                fprintf(stderr,"str_echo2: Writen error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Writen error socket: %d\n",errno);
+//close(sockfd);
+return;
             }
         }
 
@@ -377,6 +410,8 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
         //
         // Read data from pipe (parent), write to socket
         //
+        if (!pipe_from_parent)
+            exit(0);
  
         n = readline(pipe_from_parent, line, MAXLINE);
         if (n == 0) {
@@ -389,13 +424,18 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
                 //fprintf(stderr,"EAGAIN or EWOULDBLOCK\n");
             }
             else {  // Non-normal error.  Report it.
-                fprintf(stderr,"str_echo2: Readline error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Readline error pipe: %d\n",errno);
+//close(pipe_from_parent);
+return;
             }
         }
         else {  // We received some data.  Send it down the socket.
 //            fprintf(stderr,"str_echo2: %s\n",line);
+
             if (writen(sockfd, line, n) != n) {
-                fprintf(stderr,"str_echo2: Writen error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Writen error pipe: %d\n",errno);
+//close(pipe_from_parent);
+return;
             }
         }
 
@@ -459,17 +499,22 @@ int pipe_check(char *client_address) {
         //
         n = readline(p->to_parent[0], line, MAXLINE);
         if (n == 0) {
+            char timestring[101];
             pipe_object *q = pipe_head;
 
+            get_timestamp(timestring);
+
             if (p->authenticated) { 
                 fprintf(stderr,
-                    "X_spider session terminated, callsign: %s, address: %s\n",
+                    "%s X_spider session terminated, callsign: %s, address: %s\n",
+                    timestring,
                     p->callsign,
                     client_address);
             }
             else {
                 fprintf(stderr,
-                    "X_spider session terminated, unauthenticated user, address %s\n",
+                    "%s X_spider session terminated, unauthenticated user, address %s\n",
+                    timestring,
                     client_address);
             }
 
@@ -662,7 +707,7 @@ int pipe_check(char *client_address) {
 //fprintf(stderr,"Data available, sending to server\n");
 //fprintf(stderr,"\t%s\n",line);
 
-                if (writen(pipe_server_to_xastir, line, n) != n) {
+                if (writen(pipe_tcp_server_to_xastir, line, n) != n) {
                     fprintf(stderr, "pipe_check: Writen error2: %d\n", errno);
                 }
             }
@@ -674,7 +719,7 @@ int pipe_check(char *client_address) {
 
     // Check the pipe from Xastir's main thread to see if it is
     // sending us any data
-    n = readline(pipe_xastir_to_server, line, MAXLINE);
+    n = readline(pipe_xastir_to_tcp_server, line, MAXLINE);
     if (n == 0) {
         exit(0); // Connection terminated
     }
@@ -693,6 +738,22 @@ int pipe_check(char *client_address) {
 // Also send it down the socket.
         pipe_object *q = pipe_head;
 
+        // The internet protocol for sending lines is "\r\n".  Knock
+        // off any line-end characters that might be present, then
+        // add a "\r\n" combo on the end.
+        //
+        if (n > 0 && (line[n-1] == '\r' || line[n-1] == '\n')) {
+            line[n-1] = '\0';
+            n--;
+        }
+        if (n > 0 && (line[n-1] == '\r' || line[n-1] == '\n')) {
+            line[n-1] = '\0';
+            n--;
+        }
+        // Add carriage-return/linefeed onto the end
+        strncat(line, "\r\n", 2);
+        n = n+2;
+
         while (q != NULL) {
 //          fprintf(stderr,"pipe_check: %s\n",line);
 
@@ -773,18 +834,18 @@ void set_proc_title(char *fmt,...) {
     char *p;
     int i,maxlen = (LastArgv - Argv[0]) - 2;
 
-    //printf("DEBUG: maxlen: %i\n", maxlen);
+    //fprintf(stderr,"DEBUG: maxlen: %i\n", maxlen);
 
     va_start(msg,fmt);
 
     memset(statbuf, 0, sizeof(statbuf));
-    vsnprintf(statbuf, sizeof(statbuf), fmt, msg);
+    xastir_vsnprintf(statbuf, sizeof(statbuf), fmt, msg);
 
     va_end(msg);
 
     i = strlen(statbuf);
 
-    snprintf(Argv[0], maxlen, "%s", statbuf);
+    xastir_snprintf(Argv[0], maxlen, "%s", statbuf);
     p = &Argv[0][i];
   
     while(p < LastArgv)
@@ -822,8 +883,12 @@ void set_proc_title(char *fmt,...) {
 // socket and exit from the child process.  Notify the main process
 // as well?
 //
-//int main(int argc, char *argv[]) {
-void Server(int argc, char *argv[], char *envp[]) {
+#ifdef STANDALONE_PROGRAM
+int main(int argc, char *argv[]) {
+#else   // !STANDALONE_PROGRAM 
+void TCP_Server(int argc, char *argv[], char *envp[]) {
+#endif  // STANDALONE_PROGRAM
+
     int sockfd, newsockfd, childpid;
     socklen_t clilen;
     struct sockaddr_in cli_addr, serv_addr;
@@ -831,6 +896,7 @@ void Server(int argc, char *argv[], char *envp[]) {
     int sendbuff;
     int pipe_to_parent; /* symbolic names to reduce confusion */
     int pipe_from_parent;
+    char timestring[101];
 
     
     // Open a TCP listening socket
@@ -941,7 +1007,10 @@ void Server(int argc, char *argv[], char *envp[]) {
         p->authenticated = 0;
         p->callsign[0] = '\0';
 
-        fprintf(stderr,"X_spider client connected from address %s\n",
+        get_timestamp(timestring);
+
+        fprintf(stderr,"%s X_spider client connected from address %s\n",
+            timestring,
             inet_ntoa(cli_addr.sin_addr));
  
         // Link it into the head of the chain.
@@ -984,6 +1053,12 @@ void Server(int argc, char *argv[], char *envp[]) {
             // child process
             //
 
+
+            // Go back to default signal handler instead of calling
+            // restart() on SIGHUP
+            (void) signal(SIGHUP,SIG_DFL);
+
+
 /*
             fprintf(stderr,
                 "Client address: %s\n",
@@ -1000,7 +1075,7 @@ void Server(int argc, char *argv[], char *envp[]) {
                 "x-spider client @",
                 inet_ntoa(cli_addr.sin_addr),
                 "(xastir)");
-            //printf("DEBUG: %s\n", Argv[0]);
+            //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
 #endif  // __linux__
 
 // It'd be very cool here to include the IP address of the remote
@@ -1062,6 +1137,252 @@ finis:
 
 
 
+// Send a nack back to the xastir_udp_client program 
+void send_udp_nack(int sock, struct sockaddr_in from, int fromlen) {
+    int n;
+
+    n = sendto(sock,
+        "NACK", // Negative Acknowledgment
+        5,
+        0,
+        (struct sockaddr *)&from,
+        fromlen);
+    if (n < 0) {
+        fprintf(stderr, "Error: sendto");
+    }
+}
+ 
+
+
+
+
+// Create a UDP listening port.  This allows scripts and other
+// programs to inject packets into Xastir via UDP protocol.
+//
+void UDP_Server(int argc, char *argv[], char *envp[]) {
+    int sock, length, fromlen, n1, n2;
+    struct sockaddr_in server;
+    struct sockaddr_in from;
+    char buf[1024];
+    char buf2[512];
+    char *callsign;
+    short passcode;
+    char *cptr[10];
+    char *message = NULL;
+    char message2[1024];
+    int send_to_inet;
+    int send_to_rf;
+
+
+    sock = socket(AF_INET, SOCK_DGRAM, 0);
+
+    if (sock < 0) {
+        fprintf(stderr, "Error: Opening socket");
+        return;
+    }
+
+    length = sizeof(server);
+    memset(&server, 0, length);
+
+    server.sin_family = AF_INET;
+    server.sin_addr.s_addr = INADDR_ANY;
+    server.sin_port = htons(SERV_UDP_PORT);
+
+    if (bind(sock, (struct sockaddr *)&server, length) < 0) {
+        fprintf(stderr, "Error: Binding");
+        return;
+    }
+
+    fromlen = sizeof(struct sockaddr_in);
+
+    while (1) {
+        n1 = recvfrom(sock,
+            buf,
+            1024,
+            0,
+            (struct sockaddr *)&from, &fromlen);
+        if (n1 < 0) {
+            fprintf(stderr, "Error: recvfrom");
+        }
+        else if (n1 == 0) {
+            continue;
+        }
+        else {
+            buf[n1] = '\0';    // Terminate the buffer
+        }
+
+fprintf(stderr, "Received datagram: %s", buf);
+
+
+        send_to_inet = 0;
+        send_to_rf = 0;
+
+
+        //
+        // Authenticate the packet.  First line should contain:
+        //
+        //      callsign,passcode,[TO_RF],[TO_INET]
+        //
+        // The second line should contain the APRS packet
+        //
+
+        // Copy the entire buffer so that we can modify it
+        xastir_snprintf(buf2, sizeof(buf2), "%s", buf);
+        split_string(buf2, cptr, 10);
+
+        if (cptr[0] == NULL || cptr[0][0] == '\0') {    // callsign
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+        callsign = cptr[0];
+
+        if (cptr[1] == NULL || cptr[1][0] == '\0') {    // passcode
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+        passcode = atoi(cptr[1]);
+
+fprintf(stderr,"x_spider udp:  user:%s  pass:%d\n", callsign, passcode);
+
+        if (checkHash(callsign, passcode)) {
+            // Authenticate the pipe.  It is now allowed to send
+            // to the upstream server.
+            //fprintf(stderr,
+            //    "x_spider: Authenticated user %s\n",
+            //    callsign);
+        }
+        else {
+            fprintf(stderr,
+                "X_spider: Bad authentication, user %s, pass %d\n",
+                callsign,
+                passcode);
+            fprintf(stderr,
+                "UDP Packet: %s\n",
+                buf);
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+
+        // Here's where we would look for the optional flags in the
+        // first line.  Here we implement these flags:
+        //      -identify
+        //      -to_rf
+        //      -to_inet
+
+
+        // Look for the "-identify" flag in the UDP packet
+        //
+        if (strstr(buf, "-identify")) {
+            
+            // Send the callsign back to the xastir_udp_client
+            // program
+            n1 = sendto(sock,
+                my_callsign,
+                strlen(my_callsign)+1,
+                0,
+                (struct sockaddr *)&from,
+                fromlen);
+            if (n1 < 0) {
+                fprintf(stderr, "Error: sendto");
+            }
+            continue;
+        }
+
+
+        // Look for the "-to_inet" flag in the UDP packet
+        //
+        if (strstr(buf, "-to_inet")) {
+//fprintf(stderr,"Sending to INET\n");
+            send_to_inet++;
+        }
+
+
+        // Look for the "-to_rf" flag in the UDP packet
+        //
+        if (strstr(buf, "-to_rf")) {
+//fprintf(stderr,"Sending to local RF\n");
+            send_to_rf++;
+        }
+
+
+        // Now snag the text message from the second line using the
+        // original buffer.  Look for the first '\n' character which
+        // is just before the text message itself.
+        message = strchr(buf, '\n');
+        message++;  // Point to the first char after the '\n'
+
+        if (message == NULL || message[0] == '\0') {
+//fprintf(stderr,"Empty message field\n");
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+//fprintf(stderr,"Message:  %s", message);
+
+        xastir_snprintf(message2,
+            sizeof(message2),
+            "%s%s%s",
+            (send_to_inet) ? "TO_INET," : "",
+            (send_to_rf) ? "TO_RF," : "",
+            message);
+
+//fprintf(stderr,"Message2: %s", message2);
+
+
+
+//
+//
+// NOTE:
+// Should we refuse to send the message on if "callsign" and the
+// FROM callsign in the packet don't match?
+//
+// Should we change to third-party format if "my_callsign" and the
+// FROM callsign in the packet don't match?
+//
+// Require all three callsigns to match?
+//
+//
+
+
+
+        n1 = strlen(message);
+        n2 = strlen(message2);
+
+
+        // Send to Xastir udp pipe
+        //
+//fprintf(stderr,"Sending to Xastir itself\n");
+        if (writen(pipe_udp_server_to_xastir, message2, n2) != n2) {
+            fprintf(stderr,"UDP_Server: Writen error1: %d\n", errno);
+        }
+
+        // Send to the x_spider TCP server, so it can go to all
+        // connected TCP clients
+//fprintf(stderr,"Sending to TCP clients\n");
+        if (writen(pipe_xastir_to_tcp_server, message, n1) != n1) {
+            fprintf(stderr, "UDP_Server: Writen error2: %d\n", errno);
+        }
+ 
+        // Send an ACK back to the xastir_udp_client program 
+        n1 = sendto(sock,
+            "ACK",  // Acknowledgment.  Good UDP packet.
+            4,
+            0,
+            (struct sockaddr *)&from,
+            fromlen);
+        if (n1 < 0) {
+            fprintf(stderr, "Error: sendto");
+        }
+    }
+}
+
+
+
+
+
 // Function used to start a separate process for the server.  This
 // way the server can be running concurrently with the main part of
 // Xastir.
@@ -1072,36 +1393,37 @@ finis:
 // integrated with Xastir, instead of having to have a socket to
 // communicate between Xastir and the server.
 //
-int Fork_server(int argc, char *argv[], char *envp[]) {
+#ifndef STANDALONE_PROGRAM
+int Fork_TCP_server(int argc, char *argv[], char *envp[]) {
     int childpid;
 
 
     // Allocate a pipe before we fork.
     //
-    xastir_pipe = (pipe_object *)malloc(sizeof(pipe_object));
-    if (xastir_pipe == NULL) {
+    xastir_tcp_pipe = (pipe_object *)malloc(sizeof(pipe_object));
+    if (xastir_tcp_pipe == NULL) {
         fprintf(stderr,"x_spider: Couldn't malloc pipe_object\n");
         return(0);
     }
 
-    if (pipe(xastir_pipe->to_child) < 0 || pipe(xastir_pipe->to_parent) < 0) {
+    if (pipe(xastir_tcp_pipe->to_child) < 0 || pipe(xastir_tcp_pipe->to_parent) < 0) {
         fprintf(stderr,"x_spider: Can't create pipes\n");
-        free(xastir_pipe);    // Free the malloc'd memory.
+        free(xastir_tcp_pipe);    // Free the malloc'd memory.
         return(0);
     }
  
-    xastir_pipe->authenticated = 1;
-    xastir_pipe->callsign[0] = '\0';
+    xastir_tcp_pipe->authenticated = 1;
+    xastir_tcp_pipe->callsign[0] = '\0';
  
     if ( (childpid = fork()) < 0) {
-        fprintf(stderr,"Fork_server: Fork error\n");
+        fprintf(stderr,"Fork_TCP_server: Fork error\n");
 
         // Close pipes
-        close(xastir_pipe->to_child[0]);
-        close(xastir_pipe->to_child[1]);
-        close(xastir_pipe->to_parent[0]);
-        close(xastir_pipe->to_parent[1]);
-        free(xastir_pipe);    // Free the malloc'd memory.
+        close(xastir_tcp_pipe->to_child[0]);
+        close(xastir_tcp_pipe->to_child[1]);
+        close(xastir_tcp_pipe->to_parent[0]);
+        close(xastir_tcp_pipe->to_parent[1]);
+        free(xastir_tcp_pipe);    // Free the malloc'd memory.
         return(0);
     }
     else if (childpid == 0) {
@@ -1110,63 +1432,68 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
         //
 
 
+        // Go back to default signal handler instead of calling
+        // restart() on SIGHUP
+        (void) signal(SIGHUP,SIG_DFL);
+
+
         // Change the name of the new child process.  So far this
         // only works for "ps" listings, not for "top".  This code
         // only works on Linux.  For BSD use setproctitle(3), NetBSD
         // can use setprogname(2).
 #ifdef __linux__
         init_set_proc_title(argc, argv, envp);
-        set_proc_title("%s", "x-spider daemon (xastir)");
-        //printf("DEBUG: %s\n", Argv[0]);
+        set_proc_title("%s", "x-spider TCP daemon (xastir)");
+        //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
 #endif  // __linux__
  
 
-        close(xastir_pipe->to_child[1]);  // Close write end of pipe
-        close(xastir_pipe->to_parent[0]); // Close read end of pipe
+        close(xastir_tcp_pipe->to_child[1]);  // Close write end of pipe
+        close(xastir_tcp_pipe->to_parent[0]); // Close read end of pipe
 
         // Assign the global variables
-        pipe_server_to_xastir = xastir_pipe->to_parent[1];
-        pipe_xastir_to_server = xastir_pipe->to_child[0];
+        pipe_tcp_server_to_xastir = xastir_tcp_pipe->to_parent[1];
+        pipe_xastir_to_tcp_server = xastir_tcp_pipe->to_child[0];
 
         // Set read-end of pipe to be non-blocking.
         //
-        if (fcntl(pipe_xastir_to_server, F_SETFL, O_NONBLOCK) < 0) {
-            fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_server non-blocking\n");
+        if (fcntl(pipe_xastir_to_tcp_server, F_SETFL, O_NONBLOCK) < 0) {
+            fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_tcp_server non-blocking\n");
         }
 
         // Go into an infinite loop here which restarts the
         // listening process whenever it dies.
         //
 //        while (1) {
-//            fprintf(stderr,"Starting Server...\n");
+//            fprintf(stderr,"Starting TCP_Server...\n");
 
-            Server(argc, argv, envp);
+            TCP_Server(argc, argv, envp);
  
-//            fprintf(stderr,"Server process died.\n");
+//            fprintf(stderr,"TCP_Server process died.\n");
 //        }
     }
     //
     // Parent process
     //
 
-    close(xastir_pipe->to_parent[1]); // Close write end of pipe
-    close(xastir_pipe->to_child[0]);  // Close read end of pipe
+    close(xastir_tcp_pipe->to_parent[1]); // Close write end of pipe
+    close(xastir_tcp_pipe->to_child[0]);  // Close read end of pipe
 
     // Assign the global variables so that Xastir itself will know
     // how to talk to the pipes
-    pipe_server_to_xastir = xastir_pipe->to_parent[0];
-    pipe_xastir_to_server = xastir_pipe->to_child[1];
+    pipe_tcp_server_to_xastir = xastir_tcp_pipe->to_parent[0];
+    pipe_xastir_to_tcp_server = xastir_tcp_pipe->to_child[1];
 
     // Set read-end of pipe to be non-blocking.
     //
-    if (fcntl(pipe_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
-        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_server_to_xastir non-blocking\n");
+    if (fcntl(pipe_tcp_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
+        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_tcp_server_to_xastir non-blocking\n");
     }
 
 //    // Set write-end of pipe to be non-blocking.
 //    //
-//    if (fcntl(pipe_xastir_to_server, F_SETFL, O_NONBLOCK) < 0) {
-//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_server non-blocking\n");
+//    if (fcntl(pipe_xastir_to_tcp_server, F_SETFL, O_NONBLOCK) < 0) {
+//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_tcp_server non-blocking\n");
 //    }
 
     // We don't need to do anything here except return back to the
@@ -1175,5 +1502,122 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
     // when Xastir quits or segfaults.
     return(childpid);   // Really the parent PID in this case
 }
+#endif  // STANDALONE_PROGRAM
 
 
+
+
+
+int Fork_UDP_server(int argc, char *argv[], char *envp[]) {
+    int childpid;
+
+
+    // Allocate a pipe before we fork.
+    //
+    xastir_udp_pipe = (pipe_object *)malloc(sizeof(pipe_object));
+    if (xastir_udp_pipe == NULL) {
+        fprintf(stderr,"x_spider: Couldn't malloc pipe_object\n");
+        return(0);
+    }
+
+    if (pipe(xastir_udp_pipe->to_child) < 0 || pipe(xastir_udp_pipe->to_parent) < 0) {
+        fprintf(stderr,"x_spider: Can't create pipes\n");
+        free(xastir_udp_pipe);    // Free the malloc'd memory.
+        return(0);
+    }
+ 
+    xastir_udp_pipe->authenticated = 1;
+    xastir_udp_pipe->callsign[0] = '\0';
+ 
+    if ( (childpid = fork()) < 0) {
+        fprintf(stderr,"Fork_UDP_server: Fork error\n");
+
+        // Close pipes
+        close(xastir_udp_pipe->to_child[0]);
+        close(xastir_udp_pipe->to_child[1]);
+        close(xastir_udp_pipe->to_parent[0]);
+        close(xastir_udp_pipe->to_parent[1]);
+        free(xastir_udp_pipe);    // Free the malloc'd memory.
+        return(0);
+    }
+    else if (childpid == 0) {
+        //
+        // Child process
+        //
+
+
+        // Go back to default signal handler instead of calling
+        // restart() on SIGHUP
+        (void) signal(SIGHUP,SIG_DFL);
+
+
+        // Change the name of the new child process.  So far this
+        // only works for "ps" listings, not for "top".  This code
+        // only works on Linux.  For BSD use setproctitle(3), NetBSD
+        // can use setprogname(2).
+#ifdef __linux__
+        init_set_proc_title(argc, argv, envp);
+        set_proc_title("%s", "x-spider UDP daemon (xastir)");
+        //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
+#endif  // __linux__
+ 
+
+        close(xastir_udp_pipe->to_child[1]);  // Close write end of pipe
+        close(xastir_udp_pipe->to_parent[0]); // Close read end of pipe
+
+        // Assign the global variables
+        pipe_udp_server_to_xastir = xastir_udp_pipe->to_parent[1];
+        pipe_xastir_to_udp_server = xastir_udp_pipe->to_child[0];
+
+        // Set read-end of pipe to be non-blocking.
+        //
+//        if (fcntl(pipe_xastir_to_udp_server, F_SETFL, O_NONBLOCK) < 0) {
+//            fprintf(stderr,
+//                "x_spider: Couldn't set read-end of pipe_xastir_to_udp_server non-blocking\n");
+//        }
+
+        // Go into an infinite loop here which restarts the
+        // listening process whenever it dies.
+        //
+//        while (1) {
+//            fprintf(stderr,"Starting UDP_Server...\n");
+
+            UDP_Server(argc, argv, envp);
+ 
+            fprintf(stderr,"UDP_Server process died.\n");
+//        }
+    }
+    //
+    // Parent process
+    //
+
+    close(xastir_udp_pipe->to_parent[1]); // Close write end of pipe
+    close(xastir_udp_pipe->to_child[0]);  // Close read end of pipe
+
+    // Assign the global variables so that Xastir itself will know
+    // how to talk to the pipes
+    pipe_udp_server_to_xastir = xastir_udp_pipe->to_parent[0];
+    pipe_xastir_to_udp_server = xastir_udp_pipe->to_child[1];
+
+
+    // Set read-end of pipe to be non-blocking.
+    //
+    if (fcntl(pipe_udp_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
+        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_udp_server_to_xastir non-blocking\n");
+    }
+
+//    // Set write-end of pipe to be non-blocking.
+//    //
+//    if (fcntl(pipe_xastir_to_udp_server, F_SETFL, O_NONBLOCK) < 0) {
+//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_udp_server non-blocking\n");
+//    }
+
+
+    // We don't need to do anything here except return back to the
+    // calling routine with the PID of the new server process, so
+    // that it can request the server and all it's children to quit
+    // when Xastir quits or segfaults.
+    return(childpid);   // Really the parent PID in this case
+}
+
+ 
